"""
Interface en ligne de commande pour flumel:
* config : création assistée du fichier de conf
* page : génération de la page dédiée de l’instance
* nlp : téléchargement du corpus pour NLP
* init : execute les 3 tâches précédentes à la suite

Pour un premier lancement, utiliser "init"
"""

import argparse


def main():
    """init | config | page | nlp"""
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('command', help=main.__doc__)
    args = parser.parse_args()
    available_commands = {
        'init': init,
        'config': generate_config,
        'page': generate_page,
        'nlp': download_corpus
    }
    try:
        available_commands[args.command]()
    except KeyError:
        print('⚠ Commande inconnue, cli.py --help pour afficher l’aide')


def user_input(label='Poursuivre (ctr+c pour annuler)', default=''):
    """
    Simplifie la récupération des variables pour la configuration
    Affiche :
        Question [valeur par défaut] ?
    Retourne :
        Valeur saisie par l’utilisateur ou la valeur par défaut
    Utilisé sans paramètre permet de faire une pause dans l'affichage
    et demander à l'utilisateur de poursuivre
    """
    question = '{label} [{default}] ?'.format(label=label, default=default)
    return str(input(question) or default)


def init():
    """
    Initialisation de l'instance
    Execute generate_config, download_corpus & generate_page
    """
    print(init.__doc__)
    user_input()
    generate_config()
    download_corpus()
    generate_page()


def generate_config():
    """
    Génération du fichier de configuration

    Quand il n’y a pas de valeur par défaut (entre crochets) dans la question
    il faut impérativement répondre quelque chose.

    Une fois généré, le fichier flumel.cfg peut être modifié.
    """
    import configparser
    config = configparser.ConfigParser()
    config.read('flumel.cfg')
    print(generate_config.__doc__)
    user_input()

    try:
        config.add_section('Bot')
    except configparser.DuplicateSectionError:
        print('Erreur : fichier flumel.cfg déjà présent')
        return False

    print('Réglages du bot')
    config.set('Bot', 'Email', user_input('Adresse email du bot'))
    config.set('Bot', 'Login', user_input('Identifiant email'))
    config.set('Bot', 'Password', user_input('Mot de passe email'))

    print('Réglages IMAP')
    config.add_section('IMAP')
    config.set('IMAP', 'Host', user_input('Adresse du serveur IMAP'))
    config.set('IMAP', 'Port', user_input('Port (465 SSL, 587 StartTLS)',
                                          '465'))

    print('Réglages SMTP')
    config.add_section('SMTP')
    config.set('SMTP', 'Host', user_input('Adresse du serveur SMTP'))
    config.set('SMTP', 'Port', user_input('Port (993 SSL, 143 StartTLS)',
                                          '993'))

    print('Commandes')
    config.add_section('Keywords')
    config.set('Keywords', 'Subscribe',
               user_input('Pour s’abonner', 'Abonnement'))
    config.set('Keywords', 'Unsubscribe',
               user_input('Pour se désabonner', 'Désabonnement'))

    print('Base de données')
    config.add_section('DB')
    config.set('DB', 'Path',
               user_input('Nom et chemin de la base', 'flumel.sqlite'))
    config.set('DB', 'Debug', user_input('Debug activé', 'no'))

    print('Tâches')
    config.add_section('Queue')
    config.set('Queue', 'Path',
               user_input('Nom et chemin de la base', 'huey.sqlite'))
    config.set(
        'Queue', 'Management',
        user_input(
            'Fréquence de vérification IMAP (*/n = toutes les n minutes)',
            '*/3'))
    config.set(
        'Queue', 'Subscriptions',
        user_input(
            'Fréquence de vérification des flux (*/n = toutes les n heures)',
            '*/'))

    print('Journalisation')
    config.add_section('Logging')
    config.set('Logging', 'Path',
               user_input('Nom et chemin du journal', 'flumel.log'))
    config.set('Logging', 'Level',
               user_input('Niveau (DEBUG, INFO, WARN, ERROR, CRITICAL)',
                          'INFO'))

    with open('flumel.cfg', 'w') as f:
        config.write(f)

    print('✓ flumel.cfg généré !')


def generate_page():
    """
    Création de la page dédiée de l’instance
    """
    print(generate_page.__doc__)
    user_input()
    from .tasks import page
    page()
    print('✓ index.html créé !')


def download_corpus():
    """
    Téléchargement du corpus punkt pour nltk
    """
    print(download_corpus.__doc__)
    user_input()
    import nltk
    nltk.download('punkt')
    print('✓ Données téléchargées !')


if __name__ == "__main__":
    main()



"""
Boite à outils
"""
import imaplib
import smtplib
from email import message_from_bytes
from email.header import decode_header, make_header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import html2text
import pkg_resources

from . import settings


def sendmail(recipient, subject, html, **kwargs):
    """
    Envoi d'un email multipart
    Le corps du message est envoyé en html et sa conversion en texte brut par html2text
    Les kwargs permettent de personnaliser les entêtes du mail X-Flumel
    par ex. Title="Test" -> X-Flumel-Title = "Test"
    """
    server = SMTP()
    message = MIMEMultipart('alternative')
    message['Subject'] = subject
    message['From'] = settings.BOT_EMAIL
    message['To'] = recipient
    if kwargs:
        for key, value in kwargs.items():
            message['X-Flumel-' + key] = value

    text = html2text.html2text(html)
    message.attach(MIMEText(text, 'plain'))
    message.attach(MIMEText(html, 'html'))

    server.sendmail([recipient], message.as_string())
    server.close()


def format_template(name, **kwargs):
    """
    `format` le contenu du template `name` avec kwargs
    pkg_resources.resource_string est utilisé pour récupérer le path de flumel
    """
    path = '/'.join(['templates', name])
    html = pkg_resources.resource_string(__name__, path).decode("utf-8")
    return html.format(**kwargs)


def decode_email_header(value):
    """
    Les headers des emails sont encodés comme des sagouins
    https://docs.python.org/3/library/email.header.html#email.header.decode_header
    @see https://stackoverflow.com/a/12071098

    """
    return str(make_header(decode_header(value))).strip()


class IMAP:
    """
    Interface simplifiée pour imaplib.IMAP4
    La connexion est en SSL si BOT_IMAP_PORT == 993, sinon c'est en starttls
    """

    server = None

    def __init__(self):
        """
        Connexion + login
        """
        if settings.BOT_IMAP_PORT == 993:
            self.server = imaplib.IMAP4_SSL(settings.BOT_IMAP_HOST)
        else:
            self.server = imaplib.IMAP4(settings.BOT_IMAP_HOST)
            self.server.starttls()
        self.server.login(settings.BOT_LOGIN, settings.BOT_PASSWORD)

    @property
    def inbox(self):
        """
        Récupération de la liste des messages dans l’inbox, retourne un tableau iterable
        """
        self.server.select()  # ouverture inbox
        _, listing = self.server.search(None, 'ALL')  # (résultat, list)
        return listing[0].split()

    def get(self, number):
        """
        Retourne le message `number`
        """
        _, msg = self.server.fetch(number, '(RFC822)')  # (résultat, message)
        return message_from_bytes(msg[0][1])

    def delete(self, number):
        """
        Flag le message `number` à supprimer
        """
        return self.server.store(number, '+FLAGS', '\\Deleted')

    def purge(self):
        """
        Vide l’inbox après avoir marqué l’ensemble des messages à supprimer
        """
        for number in self.inbox:
            self.delete(number)
        return self.server.expunge()

    def close(self):
        """
        Efface les messages avec flag et clôture la connexion
        """
        self.server.select()  # command EXPUNGE only allowed in states SELECTED
        self.server.expunge()
        self.server.close()
        self.server.logout()


class SMTP:
    """
    Interface simplifiée pour smtplib.SMTP
    La connexion est en SSL si BOT_SMTP_PORT == 465, sinon c'est en starttls
    """

    server = None

    def __init__(self):
        """
        Connexion & login
        """
        if settings.BOT_SMTP_PORT == 465:
            self.server = smtplib.SMTP_SSL(settings.BOT_SMTP_HOST,
                                           settings.BOT_SMTP_PORT)
        else:
            self.server = smtplib.SMTP(settings.BOT_SMTP_HOST,
                                       settings.BOT_SMTP_PORT)
            if settings.BOT_SMTP_PORT == 587:
                self.server.starttls()
        try:
            self.server.login(settings.BOT_LOGIN, settings.BOT_PASSWORD)
        except smtplib.SMTPNotSupportedError:
            pass

    def sendmail(self, recipients, message):
        """
        Envoi du message
        """
        return self.server.sendmail(settings.BOT_EMAIL, recipients, message)

    def close(self):
        """
        Clôture de la connexion
        """
        self.server.quit()



"""
Setup

Usage :
* `python setup.py test -> unittests
"""

from setuptools import setup, find_packages

try:
    from pypandoc import convert
    README = convert('README.md', 'rst')
except ImportError:
    print('!!! pandoc manquant, la description sera en markdown')
    README = open('README.md').read()

with open('requirements.in') as requirements:
    REQUIRES = requirements.read().splitlines()

setup(
    name='flumel',
    version='0.3.0',  # managed by bumbversion
    description='Encore un moyen de recevoir ses flux RSS par email',
    author='Canarduck',
    author_email='renaud@canarduck.com',
    url='https://gitlab.com/canarduck/flumel',
    keywords='rss email',
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description=README,
    entry_points={
        'console_scripts': [
            'flumel-init=flumel.cli:init', 'flumel-page=flumel.cli:page',
            'flumel-nlp=flumel.cli:nlp', 'flumel-config=flumel.cli:config'
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Development Status :: 3 - Alpha', 'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Communications :: Email'
    ],
    test_suite='tests')
